'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.preparePaginationResolver = preparePaginationResolver;

var _graphqlCompose = require('graphql-compose');

var _paginationType = require('./types/paginationType');

var _paginationType2 = _interopRequireDefault(_paginationType);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign, no-use-before-define */

const DEFAULT_PER_PAGE = 20;

function preparePaginationResolver(typeComposer, opts) {
  if (!(typeComposer instanceof _graphqlCompose.TypeComposer)) {
    throw new Error('First arg for prepareConnectionResolver() should be instance of TypeComposer');
  }

  if (!opts.countResolverName) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.countResolverName`.');
  }
  const countResolver = typeComposer.getResolver(opts.countResolverName);
  if (!countResolver) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.countResolverName}' ` + 'due opts.countResolverName.');
  }
  const countResolve = countResolver.getResolve();

  if (!opts.findResolverName) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.findResolverName`.');
  }
  const findManyResolver = typeComposer.getResolver(opts.findResolverName);
  if (!findManyResolver) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.findResolverName}' ` + 'due opts.countResolverName.');
  }
  const findManyResolve = findManyResolver.getResolve();

  const additionalArgs = {};
  if (findManyResolver.hasArg('filter')) {
    const filter = findManyResolver.getArg('filter');
    if (filter) {
      additionalArgs.filter = filter;
    }
  }
  if (findManyResolver.hasArg('sort')) {
    const sort = findManyResolver.getArg('sort');
    if (sort) {
      additionalArgs.sort = sort;
    }
  }

  return new _graphqlCompose.Resolver({
    type: (0, _paginationType2.default)(typeComposer),
    name: 'pagination',
    kind: 'query',
    args: (0, _extends3.default)({
      page: {
        type: 'Int',
        description: 'Page number for displaying'
      },
      perPage: {
        type: 'Int',
        description: '',
        defaultValue: opts.perPage || DEFAULT_PER_PAGE
      }
    }, additionalArgs),
    // eslint-disable-next-line
    resolve: (() => {
      var _ref = (0, _asyncToGenerator3.default)(function* (resolveParams) {
        let countPromise;
        let findManyPromise;
        var _resolveParams$projec = resolveParams.projection;
        const projection = _resolveParams$projec === undefined ? {} : _resolveParams$projec,
              args = resolveParams.args,
              rawQuery = resolveParams.rawQuery;

        const findManyParams = (0, _extends3.default)({}, resolveParams);

        const page = parseInt(args.page, 10) || 1;
        if (page <= 0) {
          throw new Error('Argument `page` should be positive number.');
        }
        const perPage = parseInt(args.perPage, 10) || opts.perPage || DEFAULT_PER_PAGE;
        if (perPage <= 0) {
          throw new Error('Argument `perPage` should be positive number.');
        }

        const countParams = (0, _extends3.default)({}, resolveParams, {
          rawQuery,
          args: {
            filter: (0, _extends3.default)({}, resolveParams.args.filter)
          }
        });

        if (projection.count || projection.pageInfo && (projection.pageInfo.itemCount || projection.pageInfo.pageCount)) {
          countPromise = countResolve(countParams);
        } else {
          countPromise = Promise.resolve(0);
        }

        if (projection && projection.items) {
          // combine top level projection
          // (maybe somebody add additional fields via resolveParams.projection)
          // and items (record needed fields)
          findManyParams.projection = (0, _extends3.default)({}, projection, projection.items);
        } else {
          findManyParams.projection = (0, _extends3.default)({}, projection);
        }

        const limit = perPage;
        const skip = (page - 1) * perPage;

        findManyParams.args.limit = limit + 1; // +1 document, to check next page presence
        if (skip > 0) {
          findManyParams.args.skip = skip;
        }

        // pass findMany ResolveParams to top resolver
        resolveParams.findManyResolveParams = findManyParams;
        resolveParams.countResolveParams = countParams;

        // This allows to optimize and not actually call the findMany resolver
        // if only the count is projected
        if ((projection.count || projection.pageInfo) && Object.keys(projection).length === 1) {
          findManyPromise = Promise.resolve([]);
        } else {
          findManyPromise = findManyResolve(findManyParams);
        }

        return Promise.all([findManyPromise, countPromise]).then(function ([items, count]) {
          const result = {
            count,
            items: items.length > limit ? items.slice(0, limit) : items,
            pageInfo: {
              currentPage: page,
              perPage,
              itemCount: count,
              pageCount: Math.ceil(count / perPage),
              hasPreviousPage: page > 1,
              hasNextPage: items.length > limit || page * perPage < count
            }
          };
          return result;
        });
      });

      return function resolve(_x) {
        return _ref.apply(this, arguments);
      };
    })()
  });
}