'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = preparePaginationType;

var _graphql = require('graphql-compose/lib/graphql');

var _paginationInfoType = require('./paginationInfoType');

var _paginationInfoType2 = _interopRequireDefault(_paginationInfoType);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cachedPaginationTypes = new WeakMap();
/* eslint-disable arrow-body-style */

function preparePaginationType(typeComposer) {
  const name = `${typeComposer.getTypeName()}Pagination`;
  const type = typeComposer.getType();

  if (cachedPaginationTypes.has(type)) {
    return cachedPaginationTypes.get(type);
  }

  const paginationType = new _graphql.GraphQLObjectType({
    name,
    description: 'List of items with pagination.',
    fields: () => ({
      count: {
        type: _graphql.GraphQLInt,
        description: 'Total object count.'
      },
      items: {
        type: new _graphql.GraphQLList(typeComposer.getType()),
        description: 'Array of objects.'
      },
      pageInfo: {
        type: new _graphql.GraphQLNonNull(_paginationInfoType2.default),
        description: 'Information to aid in pagination.'
      }
    })
  });

  // This is small HACK for providing to graphql-compose/src/projection.js
  // information about required fields in projection and relations
  // $FlowFixMe
  paginationType.ofType = type;

  cachedPaginationTypes.set(type, paginationType);
  return paginationType;
}